// 
// Decompiled by Procyon v0.5.36
// 

package net.minecraft.nbt;

import com.google.common.collect.Iterables;
import com.google.common.base.Splitter;
import java.util.Iterator;
import com.google.common.collect.Lists;
import java.util.List;
import java.util.Stack;
import org.apache.logging.log4j.LogManager;
import java.util.regex.Pattern;
import org.apache.logging.log4j.Logger;

public class JsonToNBT
{
    private static final Logger LOGGER;
    private static final Pattern INT_ARRAY_MATCHER;
    
    static {
        LOGGER = LogManager.getLogger();
        INT_ARRAY_MATCHER = Pattern.compile("\\[[-+\\d|,\\s]+\\]");
    }
    
    public static NBTTagCompound getTagFromJson(String jsonString) throws NBTException {
        jsonString = jsonString.trim();
        if (!jsonString.startsWith("{")) {
            throw new NBTException("Invalid tag encountered, expected '{' as first char.");
        }
        if (topTagsCount(jsonString) != 1) {
            throw new NBTException("Encountered multiple top tags, only one expected");
        }
        return (NBTTagCompound)nameValueToNBT("tag", jsonString).parse();
    }
    
    static int topTagsCount(final String str) throws NBTException {
        int i = 0;
        boolean flag = false;
        final Stack<Character> stack = new Stack<Character>();
        for (int j = 0; j < str.length(); ++j) {
            final char c0 = str.charAt(j);
            if (c0 == '\"') {
                if (isCharEscaped(str, j)) {
                    if (!flag) {
                        throw new NBTException("Illegal use of \\\": " + str);
                    }
                }
                else {
                    flag = !flag;
                }
            }
            else if (!flag) {
                if (c0 != '{' && c0 != '[') {
                    if (c0 == '}' && (stack.isEmpty() || stack.pop() != '{')) {
                        throw new NBTException("Unbalanced curly brackets {}: " + str);
                    }
                    if (c0 == ']' && (stack.isEmpty() || stack.pop() != '[')) {
                        throw new NBTException("Unbalanced square brackets []: " + str);
                    }
                }
                else {
                    if (stack.isEmpty()) {
                        ++i;
                    }
                    stack.push(c0);
                }
            }
        }
        if (flag) {
            throw new NBTException("Unbalanced quotation: " + str);
        }
        if (!stack.isEmpty()) {
            throw new NBTException("Unbalanced brackets: " + str);
        }
        if (i == 0 && !str.isEmpty()) {
            i = 1;
        }
        return i;
    }
    
    static Any joinStrToNBT(final String... args) throws NBTException {
        return nameValueToNBT(args[0], args[1]);
    }
    
    static Any nameValueToNBT(final String key, String value) throws NBTException {
        value = value.trim();
        if (value.startsWith("{")) {
            value = value.substring(1, value.length() - 1);
            final Compound jsontonbt$compound = new Compound(key);
            while (value.length() > 0) {
                final String s1 = nextNameValuePair(value, true);
                if (s1.length() > 0) {
                    final boolean flag1 = false;
                    jsontonbt$compound.tagList.add(getTagFromNameValue(s1, false));
                }
                if (value.length() < s1.length() + 1) {
                    break;
                }
                final char c1 = value.charAt(s1.length());
                if (c1 != ',' && c1 != '{' && c1 != '}' && c1 != '[' && c1 != ']') {
                    throw new NBTException("Unexpected token '" + c1 + "' at: " + value.substring(s1.length()));
                }
                value = value.substring(s1.length() + 1);
            }
            return jsontonbt$compound;
        }
        if (value.startsWith("[") && !JsonToNBT.INT_ARRAY_MATCHER.matcher(value).matches()) {
            value = value.substring(1, value.length() - 1);
            final List jsontonbt$list = new List(key);
            while (value.length() > 0) {
                final String s2 = nextNameValuePair(value, false);
                if (s2.length() > 0) {
                    final boolean flag2 = true;
                    jsontonbt$list.tagList.add(getTagFromNameValue(s2, true));
                }
                if (value.length() < s2.length() + 1) {
                    break;
                }
                final char c2 = value.charAt(s2.length());
                if (c2 != ',' && c2 != '{' && c2 != '}' && c2 != '[' && c2 != ']') {
                    throw new NBTException("Unexpected token '" + c2 + "' at: " + value.substring(s2.length()));
                }
                value = value.substring(s2.length() + 1);
            }
            return jsontonbt$list;
        }
        return new Primitive(key, value);
    }
    
    private static Any getTagFromNameValue(final String str, final boolean isArray) throws NBTException {
        final String s = locateName(str, isArray);
        final String s2 = locateValue(str, isArray);
        return joinStrToNBT(s, s2);
    }
    
    private static String nextNameValuePair(final String str, final boolean isCompound) throws NBTException {
        int i = getNextCharIndex(str, ':');
        final int j = getNextCharIndex(str, ',');
        if (isCompound) {
            if (i == -1) {
                throw new NBTException("Unable to locate name/value separator for string: " + str);
            }
            if (j != -1 && j < i) {
                throw new NBTException("Name error at: " + str);
            }
        }
        else if (i == -1 || i > j) {
            i = -1;
        }
        return locateValueAt(str, i);
    }
    
    private static String locateValueAt(final String str, final int index) throws NBTException {
        final Stack<Character> stack = new Stack<Character>();
        int i = index + 1;
        boolean flag = false;
        boolean flag2 = false;
        boolean flag3 = false;
        int j = 0;
        while (i < str.length()) {
            final char c0 = str.charAt(i);
            if (c0 == '\"') {
                if (isCharEscaped(str, i)) {
                    if (!flag) {
                        throw new NBTException("Illegal use of \\\": " + str);
                    }
                }
                else {
                    flag = !flag;
                    if (flag && !flag3) {
                        flag2 = true;
                    }
                    if (!flag) {
                        j = i;
                    }
                }
            }
            else if (!flag) {
                if (c0 != '{' && c0 != '[') {
                    if (c0 == '}' && (stack.isEmpty() || stack.pop() != '{')) {
                        throw new NBTException("Unbalanced curly brackets {}: " + str);
                    }
                    if (c0 == ']' && (stack.isEmpty() || stack.pop() != '[')) {
                        throw new NBTException("Unbalanced square brackets []: " + str);
                    }
                    if (c0 == ',' && stack.isEmpty()) {
                        return str.substring(0, i);
                    }
                }
                else {
                    stack.push(c0);
                }
            }
            if (!Character.isWhitespace(c0)) {
                if (!flag && flag2 && j != i) {
                    return str.substring(0, j + 1);
                }
                flag3 = true;
            }
            ++i;
        }
        return str.substring(0, i);
    }
    
    private static String locateName(String str, final boolean isArray) throws NBTException {
        if (isArray) {
            str = str.trim();
            if (str.startsWith("{") || str.startsWith("[")) {
                return "";
            }
        }
        final int i = getNextCharIndex(str, ':');
        if (i != -1) {
            return str.substring(0, i).trim();
        }
        if (isArray) {
            return "";
        }
        throw new NBTException("Unable to locate name/value separator for string: " + str);
    }
    
    private static String locateValue(String str, final boolean isArray) throws NBTException {
        if (isArray) {
            str = str.trim();
            if (str.startsWith("{") || str.startsWith("[")) {
                return str;
            }
        }
        final int i = getNextCharIndex(str, ':');
        if (i != -1) {
            return str.substring(i + 1).trim();
        }
        if (isArray) {
            return str;
        }
        throw new NBTException("Unable to locate name/value separator for string: " + str);
    }
    
    private static int getNextCharIndex(final String str, final char targetChar) {
        int i = 0;
        boolean flag = true;
        while (i < str.length()) {
            final char c0 = str.charAt(i);
            if (c0 == '\"') {
                if (!isCharEscaped(str, i)) {
                    flag = !flag;
                }
            }
            else if (flag) {
                if (c0 == targetChar) {
                    return i;
                }
                if (c0 == '{' || c0 == '[') {
                    return -1;
                }
            }
            ++i;
        }
        return -1;
    }
    
    private static boolean isCharEscaped(final String str, final int index) {
        return index > 0 && str.charAt(index - 1) == '\\' && !isCharEscaped(str, index - 1);
    }
    
    abstract static class Any
    {
        protected String json;
        
        public abstract NBTBase parse() throws NBTException;
    }
    
    static class Compound extends Any
    {
        protected java.util.List<Any> tagList;
        
        public Compound(final String jsonIn) {
            this.tagList = (java.util.List<Any>)Lists.newArrayList();
            this.json = jsonIn;
        }
        
        @Override
        public NBTBase parse() throws NBTException {
            final NBTTagCompound nbttagcompound = new NBTTagCompound();
            for (final Any jsontonbt$any : this.tagList) {
                nbttagcompound.setTag(jsontonbt$any.json, jsontonbt$any.parse());
            }
            return nbttagcompound;
        }
    }
    
    static class List extends Any
    {
        protected java.util.List<Any> tagList;
        
        public List(final String json) {
            this.tagList = (java.util.List<Any>)Lists.newArrayList();
            this.json = json;
        }
        
        @Override
        public NBTBase parse() throws NBTException {
            final NBTTagList nbttaglist = new NBTTagList();
            for (final Any jsontonbt$any : this.tagList) {
                nbttaglist.appendTag(jsontonbt$any.parse());
            }
            return nbttaglist;
        }
    }
    
    static class Primitive extends Any
    {
        private static final Pattern DOUBLE;
        private static final Pattern FLOAT;
        private static final Pattern BYTE;
        private static final Pattern LONG;
        private static final Pattern SHORT;
        private static final Pattern INTEGER;
        private static final Pattern DOUBLE_UNTYPED;
        private static final Splitter SPLITTER;
        protected String jsonValue;
        
        static {
            DOUBLE = Pattern.compile("[-+]?[0-9]*\\.?[0-9]+[d|D]");
            FLOAT = Pattern.compile("[-+]?[0-9]*\\.?[0-9]+[f|F]");
            BYTE = Pattern.compile("[-+]?[0-9]+[b|B]");
            LONG = Pattern.compile("[-+]?[0-9]+[l|L]");
            SHORT = Pattern.compile("[-+]?[0-9]+[s|S]");
            INTEGER = Pattern.compile("[-+]?[0-9]+");
            DOUBLE_UNTYPED = Pattern.compile("[-+]?[0-9]*\\.?[0-9]+");
            SPLITTER = Splitter.on(',').omitEmptyStrings();
        }
        
        public Primitive(final String jsonIn, final String valueIn) {
            this.json = jsonIn;
            this.jsonValue = valueIn;
        }
        
        @Override
        public NBTBase parse() throws NBTException {
            try {
                if (Primitive.DOUBLE.matcher(this.jsonValue).matches()) {
                    return new NBTTagDouble(Double.parseDouble(this.jsonValue.substring(0, this.jsonValue.length() - 1)));
                }
                if (Primitive.FLOAT.matcher(this.jsonValue).matches()) {
                    return new NBTTagFloat(Float.parseFloat(this.jsonValue.substring(0, this.jsonValue.length() - 1)));
                }
                if (Primitive.BYTE.matcher(this.jsonValue).matches()) {
                    return new NBTTagByte(Byte.parseByte(this.jsonValue.substring(0, this.jsonValue.length() - 1)));
                }
                if (Primitive.LONG.matcher(this.jsonValue).matches()) {
                    return new NBTTagLong(Long.parseLong(this.jsonValue.substring(0, this.jsonValue.length() - 1)));
                }
                if (Primitive.SHORT.matcher(this.jsonValue).matches()) {
                    return new NBTTagShort(Short.parseShort(this.jsonValue.substring(0, this.jsonValue.length() - 1)));
                }
                if (Primitive.INTEGER.matcher(this.jsonValue).matches()) {
                    return new NBTTagInt(Integer.parseInt(this.jsonValue));
                }
                if (Primitive.DOUBLE_UNTYPED.matcher(this.jsonValue).matches()) {
                    return new NBTTagDouble(Double.parseDouble(this.jsonValue));
                }
                if ("true".equalsIgnoreCase(this.jsonValue) || "false".equalsIgnoreCase(this.jsonValue)) {
                    return new NBTTagByte((byte)(Boolean.parseBoolean(this.jsonValue) ? 1 : 0));
                }
            }
            catch (NumberFormatException var6) {
                this.jsonValue = this.jsonValue.replaceAll("\\\\\"", "\"");
                return new NBTTagString(this.jsonValue);
            }
            if (this.jsonValue.startsWith("[") && this.jsonValue.endsWith("]")) {
                final String s = this.jsonValue.substring(1, this.jsonValue.length() - 1);
                final String[] astring = (String[])Iterables.toArray(Primitive.SPLITTER.split((CharSequence)s), (Class)String.class);
                try {
                    final int[] aint = new int[astring.length];
                    for (int j = 0; j < astring.length; ++j) {
                        aint[j] = Integer.parseInt(astring[j].trim());
                    }
                    return new NBTTagIntArray(aint);
                }
                catch (NumberFormatException var7) {
                    return new NBTTagString(this.jsonValue);
                }
            }
            if (this.jsonValue.startsWith("\"") && this.jsonValue.endsWith("\"")) {
                this.jsonValue = this.jsonValue.substring(1, this.jsonValue.length() - 1);
            }
            this.jsonValue = this.jsonValue.replaceAll("\\\\\"", "\"");
            final StringBuilder stringbuilder = new StringBuilder();
            for (int i = 0; i < this.jsonValue.length(); ++i) {
                if (i < this.jsonValue.length() - 1 && this.jsonValue.charAt(i) == '\\' && this.jsonValue.charAt(i + 1) == '\\') {
                    stringbuilder.append('\\');
                    ++i;
                }
                else {
                    stringbuilder.append(this.jsonValue.charAt(i));
                }
            }
            return new NBTTagString(stringbuilder.toString());
        }
    }
}
