// 
// Decompiled by Procyon v0.5.36
// 

package net.minecraft.network.datasync;

import javax.annotation.Nullable;
import net.minecraft.block.Block;
import java.io.IOException;
import net.minecraft.network.PacketBuffer;
import java.util.UUID;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Rotations;
import net.minecraft.block.state.IBlockState;
import com.google.common.base.Optional;
import net.minecraft.item.ItemStack;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.IntIdentityHashBiMap;

public class DataSerializers
{
    private static final IntIdentityHashBiMap<DataSerializer<?>> REGISTRY;
    public static final DataSerializer<Byte> BYTE;
    public static final DataSerializer<Integer> VARINT;
    public static final DataSerializer<Float> FLOAT;
    public static final DataSerializer<String> STRING;
    public static final DataSerializer<ITextComponent> TEXT_COMPONENT;
    public static final DataSerializer<ItemStack> OPTIONAL_ITEM_STACK;
    public static final DataSerializer<Optional<IBlockState>> OPTIONAL_BLOCK_STATE;
    public static final DataSerializer<Boolean> BOOLEAN;
    public static final DataSerializer<Rotations> ROTATIONS;
    public static final DataSerializer<BlockPos> BLOCK_POS;
    public static final DataSerializer<Optional<BlockPos>> OPTIONAL_BLOCK_POS;
    public static final DataSerializer<EnumFacing> FACING;
    public static final DataSerializer<Optional<UUID>> OPTIONAL_UNIQUE_ID;
    
    static {
        REGISTRY = new IntIdentityHashBiMap<DataSerializer<?>>(16);
        BYTE = new DataSerializer<Byte>() {
            @Override
            public void write(final PacketBuffer buf, final Byte value) {
                buf.writeByte(value);
            }
            
            @Override
            public Byte read(final PacketBuffer buf) throws IOException {
                return buf.readByte();
            }
            
            @Override
            public DataParameter<Byte> createKey(final int id) {
                return new DataParameter<Byte>(id, this);
            }
        };
        VARINT = new DataSerializer<Integer>() {
            @Override
            public void write(final PacketBuffer buf, final Integer value) {
                buf.writeVarIntToBuffer(value);
            }
            
            @Override
            public Integer read(final PacketBuffer buf) throws IOException {
                return buf.readVarIntFromBuffer();
            }
            
            @Override
            public DataParameter<Integer> createKey(final int id) {
                return new DataParameter<Integer>(id, this);
            }
        };
        FLOAT = new DataSerializer<Float>() {
            @Override
            public void write(final PacketBuffer buf, final Float value) {
                buf.writeFloat(value);
            }
            
            @Override
            public Float read(final PacketBuffer buf) throws IOException {
                return buf.readFloat();
            }
            
            @Override
            public DataParameter<Float> createKey(final int id) {
                return new DataParameter<Float>(id, this);
            }
        };
        STRING = new DataSerializer<String>() {
            @Override
            public void write(final PacketBuffer buf, final String value) {
                buf.writeString(value);
            }
            
            @Override
            public String read(final PacketBuffer buf) throws IOException {
                return buf.readStringFromBuffer(32767);
            }
            
            @Override
            public DataParameter<String> createKey(final int id) {
                return new DataParameter<String>(id, this);
            }
        };
        TEXT_COMPONENT = new DataSerializer<ITextComponent>() {
            @Override
            public void write(final PacketBuffer buf, final ITextComponent value) {
                buf.writeTextComponent(value);
            }
            
            @Override
            public ITextComponent read(final PacketBuffer buf) throws IOException {
                return buf.readTextComponent();
            }
            
            @Override
            public DataParameter<ITextComponent> createKey(final int id) {
                return new DataParameter<ITextComponent>(id, this);
            }
        };
        OPTIONAL_ITEM_STACK = new DataSerializer<ItemStack>() {
            @Override
            public void write(final PacketBuffer buf, final ItemStack value) {
                buf.writeItemStackToBuffer(value);
            }
            
            @Override
            public ItemStack read(final PacketBuffer buf) throws IOException {
                return buf.readItemStackFromBuffer();
            }
            
            @Override
            public DataParameter<ItemStack> createKey(final int id) {
                return new DataParameter<ItemStack>(id, this);
            }
        };
        OPTIONAL_BLOCK_STATE = new DataSerializer<Optional<IBlockState>>() {
            @Override
            public void write(final PacketBuffer buf, final Optional<IBlockState> value) {
                if (value.isPresent()) {
                    buf.writeVarIntToBuffer(Block.getStateId((IBlockState)value.get()));
                }
                else {
                    buf.writeVarIntToBuffer(0);
                }
            }
            
            @Override
            public Optional<IBlockState> read(final PacketBuffer buf) throws IOException {
                final int i = buf.readVarIntFromBuffer();
                return (Optional<IBlockState>)((i == 0) ? Optional.absent() : Optional.of((Object)Block.getStateById(i)));
            }
            
            @Override
            public DataParameter<Optional<IBlockState>> createKey(final int id) {
                return new DataParameter<Optional<IBlockState>>(id, this);
            }
        };
        BOOLEAN = new DataSerializer<Boolean>() {
            @Override
            public void write(final PacketBuffer buf, final Boolean value) {
                buf.writeBoolean(value);
            }
            
            @Override
            public Boolean read(final PacketBuffer buf) throws IOException {
                return buf.readBoolean();
            }
            
            @Override
            public DataParameter<Boolean> createKey(final int id) {
                return new DataParameter<Boolean>(id, this);
            }
        };
        ROTATIONS = new DataSerializer<Rotations>() {
            @Override
            public void write(final PacketBuffer buf, final Rotations value) {
                buf.writeFloat(value.getX());
                buf.writeFloat(value.getY());
                buf.writeFloat(value.getZ());
            }
            
            @Override
            public Rotations read(final PacketBuffer buf) throws IOException {
                return new Rotations(buf.readFloat(), buf.readFloat(), buf.readFloat());
            }
            
            @Override
            public DataParameter<Rotations> createKey(final int id) {
                return new DataParameter<Rotations>(id, this);
            }
        };
        BLOCK_POS = new DataSerializer<BlockPos>() {
            @Override
            public void write(final PacketBuffer buf, final BlockPos value) {
                buf.writeBlockPos(value);
            }
            
            @Override
            public BlockPos read(final PacketBuffer buf) throws IOException {
                return buf.readBlockPos();
            }
            
            @Override
            public DataParameter<BlockPos> createKey(final int id) {
                return new DataParameter<BlockPos>(id, this);
            }
        };
        OPTIONAL_BLOCK_POS = new DataSerializer<Optional<BlockPos>>() {
            @Override
            public void write(final PacketBuffer buf, final Optional<BlockPos> value) {
                buf.writeBoolean(value.isPresent());
                if (value.isPresent()) {
                    buf.writeBlockPos((BlockPos)value.get());
                }
            }
            
            @Override
            public Optional<BlockPos> read(final PacketBuffer buf) throws IOException {
                return (Optional<BlockPos>)(buf.readBoolean() ? Optional.of((Object)buf.readBlockPos()) : Optional.absent());
            }
            
            @Override
            public DataParameter<Optional<BlockPos>> createKey(final int id) {
                return new DataParameter<Optional<BlockPos>>(id, this);
            }
        };
        FACING = new DataSerializer<EnumFacing>() {
            @Override
            public void write(final PacketBuffer buf, final EnumFacing value) {
                buf.writeEnumValue(value);
            }
            
            @Override
            public EnumFacing read(final PacketBuffer buf) throws IOException {
                return buf.readEnumValue(EnumFacing.class);
            }
            
            @Override
            public DataParameter<EnumFacing> createKey(final int id) {
                return new DataParameter<EnumFacing>(id, this);
            }
        };
        OPTIONAL_UNIQUE_ID = new DataSerializer<Optional<UUID>>() {
            @Override
            public void write(final PacketBuffer buf, final Optional<UUID> value) {
                buf.writeBoolean(value.isPresent());
                if (value.isPresent()) {
                    buf.writeUuid((UUID)value.get());
                }
            }
            
            @Override
            public Optional<UUID> read(final PacketBuffer buf) throws IOException {
                return (Optional<UUID>)(buf.readBoolean() ? Optional.of((Object)buf.readUuid()) : Optional.absent());
            }
            
            @Override
            public DataParameter<Optional<UUID>> createKey(final int id) {
                return new DataParameter<Optional<UUID>>(id, this);
            }
        };
        registerSerializer(DataSerializers.BYTE);
        registerSerializer(DataSerializers.VARINT);
        registerSerializer(DataSerializers.FLOAT);
        registerSerializer(DataSerializers.STRING);
        registerSerializer(DataSerializers.TEXT_COMPONENT);
        registerSerializer(DataSerializers.OPTIONAL_ITEM_STACK);
        registerSerializer(DataSerializers.BOOLEAN);
        registerSerializer(DataSerializers.ROTATIONS);
        registerSerializer(DataSerializers.BLOCK_POS);
        registerSerializer(DataSerializers.OPTIONAL_BLOCK_POS);
        registerSerializer(DataSerializers.FACING);
        registerSerializer(DataSerializers.OPTIONAL_UNIQUE_ID);
        registerSerializer(DataSerializers.OPTIONAL_BLOCK_STATE);
    }
    
    public static void registerSerializer(final DataSerializer<?> serializer) {
        DataSerializers.REGISTRY.add(serializer);
    }
    
    @Nullable
    public static DataSerializer<?> getSerializer(final int id) {
        return DataSerializers.REGISTRY.get(id);
    }
    
    public static int getSerializerId(final DataSerializer<?> serializer) {
        return DataSerializers.REGISTRY.getId(serializer);
    }
}
